package util;

import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

import javax.xml.bind.DatatypeConverter;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

public class BearerToken {
    /**
     * createToken
     *
     * @param typ                                 tokenType
     * @param kid                                 keyId
     * @param alg                                 signatureAlgorithm
     * @param sub                                 subject (HSBC Profile ID)
     * @param aud                                 audience (HSBC MKT)
     * @param ver                                 version (1.0)
     * @param requestPayloadEncryptedBase64String encryptedPayload (By HSBC Public Key and Base 64 Encoded)
     * @param payload_hash_alg                    algorithm used to hash the encrypted payload
     * @param privateKey                          clientPrivateKey (Convert from PGP to RSA)
     * @return token
     * @throws NoSuchAlgorithmException NoSuchAlgorithmException
     */

    public static String createToken(String typ, String kid, SignatureAlgorithm alg, String sub, String aud, String ver, String requestPayloadEncryptedBase64String, String payload_hash_alg,
                                     PrivateKey privateKey) throws NoSuchAlgorithmException {
        System.out.println("#3.1 Generating the JWT...");
        // Generate unique ID for this API call
        String jwtId = UUID.randomUUID().toString();
        long nowSeconds = System.currentTimeMillis() / 1000;
        String payloadHash = generateMessageDigest(requestPayloadEncryptedBase64String, payload_hash_alg);

        // JWT Structure
        Object[][] tokenExample = new String[12][];
        tokenExample[0] = new String[]{"@param", "In", "Description", "Value", "Source"};
        tokenExample[1] = new String[]{"------", "__", "-----------", "-----", "------"};
        tokenExample[2] = new String[]{"ver", "header", "Version", ver, "ClientConfig.TOKEN_VER"};
        tokenExample[3] = new String[]{"kid", "header", "Client Private Key ID (HEX)", kid, "Long.toHexString(clientPgpPrivateKey.getKeyID()).toUpperCase()"};
        tokenExample[4] = new String[]{"typ", "header", "Type", typ, "ClientConfig.TOKEN_TYP"};
        tokenExample[5] = new String[]{"alg", "header", "Signature Algorithm", alg.getValue(), "ClientConfig.TOKEN_ALG"};
        tokenExample[6] = new String[]{"sub", "claims", "Subject (API Profile ID", sub, "ClientConfig.API_PROFILE_ID"};
        tokenExample[7] = new String[]{"aud", "claims", "Audience", aud, "ClientConfig.TOKEN_AUD"};
        tokenExample[8] = new String[]{"payload_hash_alg", "claims", "Payload Hashing Algorithm", payload_hash_alg, "ClientConfig.TOKEN_PAYLOAD_HASH_ALG"};
        tokenExample[9] = new String[]{"payload_hash", "claims", "Hash of Base64 Encoded Payload", payloadHash, "generateMessageDigest(requestPayloadEncryptedBase64String, payload_hash_alg)"};
        tokenExample[10] = new String[]{"iat", "claims", "Timestamp", String.valueOf(nowSeconds), "System.currentTimeMillis() / 1000"};
        tokenExample[11] = new String[]{"jti", "claims", "Token (UUID)", jwtId, "UUID.randomUUID().toString()"};
        for (Object[] row : tokenExample) {
            System.out.format("%-18s%-7s%-34s%-66s%-66s%n", row);
        }

        Map<String, Object> header = new HashMap<String, Object>();
        header.put("typ", typ);
        header.put("kid", kid);
        header.put("alg", alg.getValue());
        header.put("ver", ver);
        System.out.println("\n#3.2 Generated JWT:\n{");
        System.out.println("  \"header\": {");
        Integer mapIndex = 0;
        for (Map.Entry<String, Object> headerEntry : header.entrySet()) {
            System.out.print("    \"" + headerEntry.getKey() + "\": \"" + headerEntry.getValue() + "\"");
            if (mapIndex < header.size() - 1) {
                System.out.println(",");
                mapIndex++;
            } else {
                System.out.println();
            }
        }
        System.out.println("  },");
        Map<String, Object> claims = new HashMap<String, Object>();
        claims.put("sub", sub);
        claims.put("iat", nowSeconds);
        claims.put("aud", aud);
        claims.put("jti", jwtId);
        claims.put("payload_hash", payloadHash);
        claims.put("payload_hash_alg", payload_hash_alg);

        System.out.println("  \"claims\": {");
        mapIndex = 0;
        for (Map.Entry<String, Object> claimsEntry : claims.entrySet()) {
            System.out.print("    \"" + claimsEntry.getKey() + "\": \"" + claimsEntry.getValue() + "\"");
            if (mapIndex < claims.size() - 1) {
                System.out.println(",");
                mapIndex++;
            } else {
                System.out.println();
            }
        }
        System.out.println("  }\n}");

        String token = Jwts.builder().setHeader(header)
                .setClaims(claims).signWith(privateKey, alg).compact();
        System.out.println("\n#3.3 Signed JWT (JWS):");
        System.out.println("JWS " + token);
        return token;
    }

    // Helper method for getting message digest
    private static String generateMessageDigest(String message, String hashingAlgorithm)
            throws NoSuchAlgorithmException {
        return DatatypeConverter.printHexBinary(MessageDigest.getInstance(hashingAlgorithm).digest(message.getBytes()))
                .toLowerCase();
    }

}
